/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#include "libtkcommon/debug.h"

#include "flash_toggle_tool.h"

using namespace TkResolver::Experimenter;

FlashToggleTool:: FlashToggleTool(const Gtk::StockID &stock_id,
				  const Gtk::IconSize &icon_size,
				  const Gdk::Color& flash_colour)
  : _normal_icon(stock_id, icon_size),
    _on(false)
{
  Gtk::StockItem stock_item;
  Gtk::Stock::lookup(stock_id, stock_item);
  Glib::ustring label(stock_item.get_label());

  int underscore = label.find("_");
  if (underscore != -1)
    {
      label.replace(underscore, 1, "");
    }

  set_label(label);



  Glib::RefPtr<Gdk::Pixbuf> normal(render_icon(stock_id, icon_size));
  Glib::RefPtr<Gdk::Pixbuf> flash(render_icon(stock_id, icon_size)->copy());

  guint32 fill_value(0);
  fill_value += (static_cast<guint32>(flash_colour.get_red_p()   * 0xFF) << 24);
  fill_value += (static_cast<guint32>(flash_colour.get_green_p() * 0xFF) << 16);
  fill_value += (static_cast<guint32>(flash_colour.get_blue_p()  * 0xFF) <<  8);
  fill_value += 0xFF;
  flash->fill(fill_value);

  normal->composite(flash,
		    0, 0, flash->get_width(), flash->get_height(),
		    0.0, 0.0, 1.0, 1.0,
		    Gdk::INTERP_NEAREST, 230);

  _flash_icon.set(flash);

  _normal_icon.show();
  _flash_icon.show();
  set_icon_widget(_normal_icon);
}

void
FlashToggleTool:: on()
{
  set_icon_widget(_flash_icon);
  _on = true;
}

void
FlashToggleTool:: off()
{
  set_icon_widget(_normal_icon);
  _on = false;
}

void
FlashToggleTool:: flash_start(unsigned long interval)
{
  on();
  _flash_signal = Glib::signal_timeout().connect(sigc::mem_fun(*this, &FlashToggleTool::signal), interval);
}

void
FlashToggleTool:: flash_stop()
{
  _flash_signal.disconnect();
  off();
}

bool
FlashToggleTool:: signal()
{
  if (_on)
    off();
  else
    on();

  return true;
}
