/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_ALGORITHM_H__
#define __TK_ALGORITHM_H__

#include <list>
#include <vector>
#include <map>

#include "libtkcommon/tkstring.h"
#include "libtkrandom/randomness.h"

namespace TkResolver {

  typedef short Offset;
  typedef long double Probability;
  typedef std::map<Offset, Probability> OffsetProbabilities;
  typedef std::vector<short> SmallNumbers;

  class Algorithm
  {
  private:
    
    // probability weights, eg, 1 2 1
    SmallNumbers        _weights;
    // offsets, eg -1 0 1
    SmallNumbers        _offsets;
    // pixel offset space, eg -1 0 0 1
    SmallNumbers        _outcome_pattern;
    unsigned            _outcome_count;
    // mapping of offsets to probabilities
    OffsetProbabilities _probabilities;
    
  public:
    /** The spec is a space-seperated list of integers.
     * It defines the probability weights that will be placed
     * on the different possible trace movements.  The
     * list contains N/2+1 elements, where N is the number of
     * possibile positions; the last element is the central
     * position.
     *
     * Eg, For an equally distributed probability, one pixel
     * in either direction, the spec would be as follows:
     *
     *  '1 1'
     *
     * which would result in the following distribution:
     *
     *      ---
     *     |XXX|  <-- previous line's pixel
     *  ---|---|---
     * |33%|33%|33%|
     *  --- --- ---
     *
     */
    Algorithm (const TkString &spec);
    
    Offset choose_offset(unsigned choice);
    
    const SmallNumbers & get_weights() const;
    const SmallNumbers & get_offsets() const;
    const SmallNumbers & get_outcome_pattern() const;
    unsigned get_outcome_count() const;
    const OffsetProbabilities & get_probabilities() const;
  };

}; /* namespace TkResolver */

#endif /* __TK_ALGORITHM_H__ */

