/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_ANALYSER_H__
#define __TK_ANALYSER_H__

#include "libtkcommon/tkstring.h"
#include "libtkcommon/exceptions.h"
#include "libtkalgorithm/algorithm.h"
#include "libtkdb/database.h"
#include "libtkresolver/types.h"

namespace TkResolver {
  namespace Analysis {

    typedef unsigned long Count;
    typedef std::map<Offset, Count> OffsetCounts;

    class NotEnoughResults : std::runtime_error
      {
      public:
	NotEnoughResults()
	  : std::runtime_error("not enough results") {}
      };
    
    class DataAdaptor
      {
      public:
	virtual Offset get_offset() = 0;
	virtual bool emptied() = 0;
      };
    
    class Analyser
      {
      private:
	OffsetProbabilities _offset_probabilities;
	OffsetCounts        _offset_counts;
	Probability         _chi_square;
	Probability         _p;
	Count               _n;
	
	void analyse (DataAdaptor & adaptor);
      public:
	Analyser(const OffsetProbabilities &offset_probabilities,
		 const Result &result_lines);
	Analyser(Db::Database & database,
		 const TkString & experiment_group,
		 bool psuedo,
		 bool control);
	Probability get_chi_square () const;
	Probability get_p () const;
	unsigned get_df () const;
	Count get_n() const;
	const OffsetCounts & get_offset_counts() const;
	const OffsetProbabilities & get_offset_probabilities() const;
      };
    
  }; /* namespace Analysis */
}; /* namespace TkResolver */

#endif /* __TK_ANALYSER_H__ */

