/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_MBS_H__
#define __TK_MBS_H__

#include <wchar.h>
#include <stdlib.h>
#include <cstring>
#include <string.h>

#include <glibmm.h>


namespace TkResolver {

  inline wchar_t * mbstowcs_alloc(const char *string)
    {
      size_t size = strlen(string) + 1;
      wchar_t * buf = static_cast<wchar_t *> (malloc (size * sizeof (wchar_t)));
      
      size = mbstowcs (buf, string, size);
      if (size == (size_t) -1)
	return NULL;
      
      buf = static_cast<wchar_t *> (realloc (buf, (size + 1) * sizeof (wchar_t)));
      return buf;
    }
  
  inline char * wcstombs_alloc(const wchar_t * string)
    {
      size_t size = wcslen(string) + 1;
      char * buf = static_cast<char *> (malloc (size * sizeof (char)));
      
      size = wcstombs (buf, string, size);
      if (size == (size_t) -1)
	return NULL;
      
      buf = static_cast<char *> (realloc (buf, (size + 1) * sizeof (char)));
      return buf;
    }
  
  inline Glib::ustring wstring_to_ustring(const std::wstring & wstr)
    {
      char * str = wcstombs_alloc(wstr.c_str());
      Glib::ustring ustr(str);
      free(str);
      return ustr;
    }

  inline std::wstring ustring_to_wstring(const Glib::ustring & ustr)
    {
      wchar_t * str = mbstowcs_alloc(ustr.c_str());
      std::wstring wstr(str);
      free(str);
      return wstr;
    }

  inline std::string wstring_to_string(const std::wstring & wstr)
    {
      char * str = wcstombs_alloc(wstr.c_str());
      std::string sstr(str);
      free(str);
      return sstr;
    }

  inline std::wstring mbs_to_wstring(const char * cstr)
    {
      wchar_t * str = mbstowcs_alloc(cstr);
      std::wstring wstr(str);
      free(str);
      return wstr;
    }
  inline std::wstring string_to_wstring(const std::string & sstr)
    {
      return mbs_to_wstring(sstr.c_str());
    }
  
  
}; /* namespace TkResolver */


#endif /* __TK_MBS_H__ */
