/* -*- c++-mode -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_DATABASE_H__
#define __TK_DATABASE_H__

#define GETTEXT_PACKAGE "TkResolver"

#include <stdexcept>

#include <mysql++/mysql++.h>

#include "libtkcommon/tkstring.h"
#include "libtkrandom/random_types.h"
#include "libtkresolver/types.h"

#define EXPERIMENT_GROUP "Test group"

namespace TkResolver {
  namespace Db {
    
    typedef my_ulonglong Id;
    typedef std::list<Id> IdList;
    typedef mysqlpp::Date Date;
    typedef mysqlpp::DateTime DateTime;
    typedef std::string Name;
    typedef std::list<Name> NameList;
    typedef mysqlpp::ColData Value;
    typedef std::map<Name, Value> Values;
    typedef mysqlpp::SQLString String;
    typedef std::map<Name, String> Parameters;
    typedef Parameters::value_type Parameter;
    typedef mysqlpp::mysql_type_info Type;
    typedef mysqlpp::Result DataSet;
    typedef mysqlpp::ResUse DataCursor;
    typedef mysqlpp::Row Row;

    String date_time_string (time_t t);
    
    class Error : public std::runtime_error
      {
      public:
	Error (mysqlpp::Connection & connection);
      };
  
    class Database
      {
      private:
	typedef std::map<Name, Type> FieldInfo;
	typedef std::map<Name, FieldInfo> TableInfo;

	mysqlpp::Connection _connection;
	
	mysqlpp::Query *    _table_info_query;
	TableInfo           _table_info;

	void create_table_info_query();
	void get_table_info(const Name & name);
	void check_table_info(const Name & name);

	Id insert_experiment(const TkString & experiment_group,
			     time_t date_time,
			     const TkString & location_name,
			     const TkString & notes,
			     unsigned width,
			     unsigned height,
			     const TkString & retriever,
			     bool retriever_is_true_random,
			     bool retreiver_specified,
			     const Random::Bits & randomness);
	void insert_experiment(const Id & experiment_id,
			       const IdList & operators);
	void insert_experiment(const Id & experiment_id,
			       const Result & result);
      public:
	Database(const TkString & hostname = "localhost",
		 const TkString & database = "psi",
		 const TkString & username = "psi",
		 const TkString & password = "isp");
	~Database();

	Type get_column_type(const Name & table_name, const Name & column_name);
	DataSet * get_table_data(const Name & table_name, const NameList & column_names);
	Id insert_table_data(const Name & table_name, const Parameters & params);

	void experiment_group_get_parameters(const TkString & group_name,
					     TkString * algorithm_spec,
					     unsigned * bit_descriptor = 0,
					     unsigned short * traces = 0,
					     unsigned * total_time = 0,
					     double * speed = 0,
					     double * horizon = 0);
	DataCursor * experiment_group_get_results(const TkString & group_name,
						  bool true_random,
						  bool control);
	

	Id insert_experiment(const TkString & experiment_group,
			     time_t date_time,
			     const TkString & location_name,
			     const TkString & notes,
			     unsigned width,
			     unsigned height,
			     const TkString & retriever,
			     bool retriever_is_true_random,
			     bool retriever_specified,
			     const Random::Bits & randomness,
			     const IdList & operators,
			     const Result & result);
        TkString choose_retriever(const TkString & group_name, const TkStringList & retrievers);
      };

  }; /* namespace Db */
}; /* namespace TkResolver */

#endif /* __TK_DATABASE_H__ */
