/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

template<typename DI>
DataSelectorList<DI>:: DataSelectorList(Db::Database * database,
					const Name & table_name,
					const NameList & selection_columns,
					Gtk::Window * parent,
					const Name & title)
  : DataSelector (database, table_name, selection_columns),
    Gtk::Frame(title),
    _parent(parent)
{
  Gtk::HBox * main_box = Gtk::manage(new Gtk::HBox);
  add(*main_box);

  //
  // Data view
  //
  _view_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  main_box->pack_start(_view_scroll, true, true);
  _view_scroll.set_shadow_type(Gtk::SHADOW_IN);

  //
  // Control buttons
  //
  Gtk::VButtonBox * buttons = Gtk::manage(new Gtk::VButtonBox(Gtk::BUTTONBOX_START));
  main_box->pack_start(*buttons, false, true);

  Gtk::Button * button = Gtk::manage(new Gtk::Button(Gtk::Stock::ADD));
  button->set_focus_on_click(false);
  button->signal_clicked().connect( sigc::mem_fun(*this, &DataSelectorList<DI>::input) );
  buttons->pack_start(*button, true, true);

}


template<typename DI>
void 
DataSelectorList<DI>:: set_display_columns(const ListColumnList & display_columns)
{
  //
  // Create the data models
  //
  set_columns(display_column_names(display_columns));
  
  _view.set_model(get_store());

  for (typename ListColumnList::const_iterator iter(display_columns.begin());
       iter != display_columns.end();
       ++iter)
    {
      Gtk::TreeViewColumn * column = Gtk::manage(new Gtk::TreeViewColumn(iter->title));
      ListColumnAdaptor adaptor(column);
      create_view_cell(*iter, adaptor);
      _view.append_column(*column);
    }
  _view_scroll.add(_view);
  
  show_all_children();
}

template<typename DI>
void
DataSelectorList<DI>:: input()
{
  DataInput * di = new DI(*_parent);

  Db::Parameters * parameters(di->get_data());
  delete di;

  if (!parameters)
    return;

  get_database()->insert_table_data(get_table_name(), *parameters);
  delete parameters;

  refresh();
}

template<typename DI>
Gtk::TreeView &
DataSelectorList<DI>:: get_view()
{
  return _view;
}

template<typename DI>
ModelIteratorList
DataSelectorList<DI>:: get_selected() const
{
  const Glib::RefPtr<const Gtk::TreeSelection> sel(_view.get_selection());
  
  if (sel->count_selected_rows() < 1)
    {
      throw NoSelection();
    }

  ModelIteratorList list;
  sel->selected_foreach_iter( sigc::bind(sigc::ptr_fun(&append_selection), &list) );

  return list;
}

template<typename DI>
void
DataSelectorList<DI>:: refreshed()
{
  
}
