/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_RANDOMNESS_H__
#define __TK_RANDOMNESS_H__

#include <vector>

#include "libtkcommon/tkstring.h"
#include "random_types.h"
#include "retriever_registry.h"
#include "retriever.h"

namespace TkResolver {

  namespace Random {

    class Randomness
      {
      private:
      typedef std::list<Retriever *> Retrievers;
      typedef enum { Stopped, Running } RunState;

      pthread_t         _join_thread;
      pthread_mutex_t   _join_lock; 
      pthread_cond_t    _join_signal;
      RunState          _run_state;
      TkString          _retriever_type;
      Retrievers        _retrievers;
      unsigned          _retrieving;
      unsigned          _retrieved;
      
      Buffer            _buffer;
      unsigned          _buffer_size;
      unsigned          _threshold; /**< In bytes (chars) >*/

      char              _randomness;
      unsigned short    _bit_index;

      Bits            * _used_bits;

      static void * low_join_run (void *);
      void * join_run ();

      void lock ();
      void unlock ();

      void append_buffer (const Buffer * buf);
      void retrieve (unsigned count);
      unsigned retrieved();
      void check_threshold ();

    public:
      /// Generate a random number between 0 and upper
      static unsigned long get_number(unsigned long upper, const TkString & retriever_type = "alea");
      
      Randomness(const TkString &retriever_type, bool deterministic, unsigned buffer_size, bool record_use = true);
      ~Randomness();

      bool get_bit();
      unsigned get_choice();
      const Bits& get_used_bits() const;

    };
    
  }; /* namespace Random */
  
}; /* namespace TkResolver */


#endif /* __TK_RANDOMNESS_H__ */
