/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#include <stdexcept>
#include <boost/assign/list_of.hpp>

#include "libtkcommon/config.h"
#include "libtkcommon/debug.h"
#include "retriever_registry.h"
#include "pseudo_random.h"
#ifdef HAVE_USB
#include "alea.h"
#endif /* HAVE_USB */

using namespace TkResolver;
using namespace Random;

const RetrieverTypeMap RetrieverRegistry::_registry =
  boost::assign::map_list_of
  (TkString("pseudo"), static_cast<RetrieverType *>(new PseudoRandomType()) )

#ifdef HAVE_USB
  (TkString("alea"),   static_cast<RetrieverType *>(new AleaType())         )
#endif /* HAVE_USB */
  ;



TkStringList
RetrieverRegistry:: get_names()
{
  TkStringList list;

  for (RetrieverTypeMap::const_iterator iter = _registry.begin();
       iter != _registry.end();
       ++iter)
    {
      list.push_back(iter->first);
    }

  return list;
}


RetrieverType *
RetrieverRegistry:: get_retriever(const TkString &type)
{
  RetrieverTypeMap::const_iterator iter(_registry.find(type));
  
  if (iter == _registry.end())
    throw std::invalid_argument("unknown retriever type");
  
  return iter->second;
}


unsigned
RetrieverRegistry:: get_preferred_block_size(const TkString &type)
{
  return get_retriever(type)->get_preferred_block_size();
}

Retriever *
RetrieverRegistry:: new_retriever (const TkString &type, unsigned count)
{
  return get_retriever(type)->new_retriever(count);
}

bool
RetrieverRegistry:: is_true_randomness(const TkString & type)
{
  return get_retriever(type)->is_true_randomness();
}

/* EOF */
